/* adc_interface.c - adc functions

Copyright (c) 2012 David Michael Betz

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/

#include <propeller.h>
#include "adc.h"

/* we need to reference this symbol to get the driver linked */
extern int _adc_driver;
int _adc_driver_loaded = (int)&_adc_driver;

extern unsigned int _load_start_adc_driver_cog[];

ADC *adcOpen(ADC *dev, int scl, int sda, int addr, int mask, int freq, uint16_t *buffer, int size)
{
    ADC_INIT init;
    
    // only allow speeds up to 400khz for now
    if (freq > 400000)
        return NULL;
    
    init.scl = scl;
    init.sda = sda;
    init.addr = addr;
    init.mask = mask;
    init.ticks_per_cycle = CLKFREQ / freq;

    dev->state.mailbox.init = &init;
    dev->state.mailbox.buffer = buffer;
    dev->state.mailbox.size = size;
    
    if ((dev->state.mailbox.lock = locknew()) == -1)
        return NULL;
    
    if ((dev->cog = cognew(_load_start_adc_driver_cog, &dev->state.mailbox)) < 0) {
        lockret(dev->state.mailbox.lock);
        return NULL;
    }
    
    while (dev->state.mailbox.cmd != ADC_CMD_IDLE)
        ;
        
    return (ADC *)dev;
}

int adcClose(ADC *dev)
{
    cogstop(dev->cog);
    lockret(dev->state.mailbox.lock);
    return 0;
}

int adcRead(ADC *dev, uint16_t *buffer, int size)
{
    int count = 0;
    while (count < size) {
        if (dev->state.mailbox.oPtr != dev->state.mailbox.iPtr) {
            while (lockset(dev->state.mailbox.lock) != 0)
                ;
            *buffer++ = dev->state.mailbox.buffer[dev->state.mailbox.oPtr++];
            if (dev->state.mailbox.oPtr >= dev->state.mailbox.size)
                dev->state.mailbox.oPtr = 0;
            lockclr(dev->state.mailbox.lock);
            ++count;
        }
    }
    return count;
}
